@@ -92,7 +92,11 @@ func (c actionTests) actionExec(t *testing.T) {
 	if err != nil {
 		t.Fatal(err)
 	}
-	defer os.RemoveAll(testdata)
+	t.Cleanup(func() {
+		if !t.Failed() {
+			os.RemoveAll(testdata)
+		}
+	})
 
 	testdataTmp := filepath.Join(testdata, "tmp")
 	if err := os.Mkdir(testdataTmp, 0o755); err != nil {
@@ -112,9 +116,10 @@ func (c actionTests) actionExec(t *testing.T) {
 	homePath := filepath.Join("/home", basename)
 
 	tests := []struct {
-		name string
-		argv []string
-		exit int
+		name        string
+		argv        []string
+		exit        int
+		wantOutputs []e2e.SingularityCmdResultOp
 	}{
 		{
 			name: "NoCommand",
@@ -220,7 +225,10 @@ func (c actionTests) actionExec(t *testing.T) {
 		},
 		{
 			name: "Home",
-			argv: []string{"--home", testdata, c.env.ImagePath, "test", "-f", tmpfile.Name()},
+			argv: []string{"--home", "/myhomeloc", c.env.ImagePath, "env"},
+			wantOutputs: []e2e.SingularityCmdResultOp{
+				e2e.ExpectOutput(e2e.RegexMatch, `\bHOME=/myhomeloc\b`),
+			},
 			exit: 0,
 		},
 		{
@@ -253,6 +261,30 @@ func (c actionTests) actionExec(t *testing.T) {
 			argv: []string{"--no-home", c.env.ImagePath, "ls", "-ld", user.Dir},
 			exit: 1,
 		},
+		{
+			name: "Hostname",
+			argv: []string{"--hostname", "whats-in-a-native-name", c.env.ImagePath, "hostname"},
+			exit: 0,
+			wantOutputs: []e2e.SingularityCmdResultOp{
+				e2e.ExpectOutput(e2e.ExactMatch, "whats-in-a-native-name"),
+			},
+		},
+		{
+			name: "ResolvConfGoogle",
+			argv: []string{"--dns", "8.8.8.8,8.8.4.4", c.env.ImagePath, "nslookup", "w3.org"},
+			exit: 0,
+			wantOutputs: []e2e.SingularityCmdResultOp{
+				e2e.ExpectOutput(e2e.RegexMatch, `^(\s*)Server:(\s+)(8\.8\.8\.8|8\.8\.4\.4)(\s*)\n`),
+			},
+		},
+		{
+			name: "ResolvConfCloudflare",
+			argv: []string{"--dns", "1.1.1.1", c.env.ImagePath, "nslookup", "w3.org"},
+			exit: 0,
+			wantOutputs: []e2e.SingularityCmdResultOp{
+				e2e.ExpectOutput(e2e.RegexMatch, `^(\s*)Server:(\s+)(1\.1\.1\.1)(\s*)\n`),
+			},
+		},
 	}
 
 	for _, tt := range tests {
@@ -263,7 +295,7 @@ func (c actionTests) actionExec(t *testing.T) {
 			e2e.WithCommand("exec"),
 			e2e.WithDir("/tmp"),
 			e2e.WithArgs(tt.argv...),
-			e2e.ExpectExit(tt.exit),
+			e2e.ExpectExit(tt.exit, tt.wantOutputs...),
 		)
 	}
 }
@@ -1134,10 +1166,11 @@ func (c actionTests) actionBinds(t *testing.T) {
 	}
 
 	tests := []struct {
-		name    string
-		args    []string
-		postRun func(*testing.T)
-		exit    int
+		name        string
+		args        []string
+		wantOutputs []e2e.SingularityCmdResultOp
+		postRun     func(*testing.T)
+		exit        int
 	}{
 		{
 			name: "NonExistentSource",
@@ -1447,6 +1480,28 @@ func (c actionTests) actionBinds(t *testing.T) {
 			postRun: checkHostDir(filepath.Join(hostWorkDir, "var_tmp", "canary/dir")),
 			exit:    0,
 		},
+		{
+			name: "IsScratchTmpfs",
+			args: []string{
+				"--scratch", "/name-of-a-scratch",
+				sandbox,
+				"mount",
+			},
+			wantOutputs: []e2e.SingularityCmdResultOp{
+				e2e.ExpectOutput(e2e.RegexMatch, `\btmpfs on /name-of-a-scratch\b`),
+			},
+			exit: 0,
+		},
+		{
+			name: "BindOverScratch",
+			args: []string{
+				"--scratch", "/name-of-a-scratch",
+				"--bind", hostCanaryDir + ":/name-of-a-scratch",
+				sandbox,
+				"test", "-f", "/name-of-a-scratch/file",
+			},
+			exit: 0,
+		},
 		{
 			name: "ScratchTmpfsBind",
 			args: []string{
@@ -1509,7 +1564,7 @@ func (c actionTests) actionBinds(t *testing.T) {
 					e2e.WithCommand("exec"),
 					e2e.WithArgs(tt.args...),
 					e2e.PostRun(tt.postRun),
-					e2e.ExpectExit(tt.exit),
+					e2e.ExpectExit(tt.exit, tt.wantOutputs...),
 				)
 			}
 		})
@@ -2413,6 +2468,26 @@ func countSquashfuseMounts(t *testing.T) int {
 	return count
 }
 
+// actionNoSetgoups checks that supplementary groups are visible, mapped to
+// nobody, in the container with --fakeroot --no-setgroups.
+func (c actionTests) actionNoSetgroups(t *testing.T) {
+	e2e.EnsureImage(t, c.env)
+
+	// Inside the e2e-tests we will be a member of our user group + single supplementary group.
+	// With `--fakeroot --no-setgroups` we should see these map to:
+	//    root nobody
+	c.env.RunSingularity(
+		t,
+		e2e.WithProfile(e2e.FakerootProfile),
+		e2e.WithCommand("exec"),
+		e2e.WithArgs("--no-setgroups", c.env.ImagePath, "sh", "-c", "groups"),
+		e2e.ExpectExit(
+			0,
+			e2e.ExpectOutput(e2e.ExactMatch, "root nobody"),
+		),
+	)
+}
+
 // E2ETests is the main func to trigger the test suite
 func E2ETests(env e2e.TestEnv) testhelper.Tests {
 	c := actionTests{
@@ -2453,6 +2528,7 @@ func E2ETests(env e2e.TestEnv) testhelper.Tests {
 		"fuse mount":                c.fuseMount,               // test fusemount option
 		"bind image":                c.bindImage,               // test bind image with --bind and --mount
 		"no-mount":                  c.actionNoMount,           // test --no-mount
+		"no-setgroups":              c.actionNoSetgroups,       // test --no-setgroups
 		"compat":                    np(c.actionCompat),        // test --compat
 		"umask":                     np(c.actionUmask),         // test umask propagation
 		"invalidRemote":             np(c.invalidRemote),       // GHSA-5mv9-q7fq-9394
@@ -2461,10 +2537,13 @@ func E2ETests(env e2e.TestEnv) testhelper.Tests {
 		//
 		// OCI Runtime Mode
 		//
-		"ociRun":     c.actionOciRun,     // singularity run --oci
-		"ociExec":    c.actionOciExec,    // singularity exec --oci
-		"ociShell":   c.actionOciShell,   // singularity shell --oci
-		"ociNetwork": c.actionOciNetwork, // singularity exec --oci --net
-		"ociBinds":   c.actionOciBinds,   // singularity exec --oci --bind / --mount
+		"ociRun":     c.actionOciRun,        // singularity run --oci
+		"ociExec":    c.actionOciExec,       // singularity exec --oci
+		"ociShell":   c.actionOciShell,      // singularity shell --oci
+		"ociSTDPIPE": c.ociSTDPipe,          // stdin/stdout pipe --oci
+		"ociNetwork": c.actionOciNetwork,    // singularity exec --oci --net
+		"ociBinds":   c.actionOciBinds,      // singularity exec --oci --bind / --mount
+		"ociIDMaps":  c.actionOciIDMaps,     // check uid/gid mapping on host for --oci as user / --fakeroot
+		"ociCompat":  np(c.actionOciCompat), // --oci equivalence to native mode --compat
 	}
 }
