@@ -18,7 +18,7 @@ import (
 
 //  NOTE
 //  ----
-//  Tests in this package/topic are run in a a mount namespace only. There is
+//  Tests in this package/topic are run in a mount namespace only. There is
 //  no PID namespace, in order that the systemd cgroups manager functionality
 //  can be exercised.
 //
@@ -54,7 +54,7 @@ func (c *ctx) instanceStats(t *testing.T, profile e2e.Profile) {
 	}{
 		{
 			name:           "basic stats create",
-			createArgs:     []string{c.env.ImagePath},
+			createArgs:     []string{"--memory", "250M", c.env.ImagePath},
 			statsErrorCode: 0,
 			startErrorCode: 0,
 		},
@@ -85,13 +85,18 @@ func (c *ctx) instanceStats(t *testing.T, profile e2e.Profile) {
 				e2e.WithCommand("instance stats"),
 				e2e.WithArgs("--no-stream", instanceName),
 				e2e.ExpectExit(tt.statsErrorCode,
-					e2e.ExpectOutput(e2e.ContainMatch, instanceName),
+					// Header (column spacing varies by content)
 					e2e.ExpectOutput(e2e.ContainMatch, "INSTANCE NAME"),
 					e2e.ExpectOutput(e2e.ContainMatch, "CPU USAGE"),
 					e2e.ExpectOutput(e2e.ContainMatch, "MEM USAGE / LIMIT"),
 					e2e.ExpectOutput(e2e.ContainMatch, "MEM %"),
 					e2e.ExpectOutput(e2e.ContainMatch, "BLOCK I/O"),
 					e2e.ExpectOutput(e2e.ContainMatch, "PIDS"),
+					e2e.ExpectOutput(e2e.ContainMatch, "PIDS"),
+					// Instance name is visible
+					e2e.ExpectOutput(e2e.ContainMatch, instanceName),
+					// Memory limit is visible
+					e2e.ExpectOutput(e2e.ContainMatch, "/ 250MiB"),
 				),
 			)
 			c.env.RunSingularity(
@@ -138,7 +143,7 @@ func (c *ctx) instanceApply(t *testing.T, profile e2e.Profile) {
 			startErrorCode: 255,
 			// e2e test currently only captures the error from the CLI process, not the error displayed by the
 			// starter process, so we check for the generic CLI error.
-			startErrorOut: "parsing error",
+			startErrorOut: "toml: expected character",
 			rootfull:      true,
 			rootless:      true,
 		},
@@ -271,7 +276,7 @@ func (c *ctx) actionApply(t *testing.T, profile e2e.Profile, imageRef string) {
 			name:            "invalid toml",
 			args:            []string{"--apply-cgroups", "testdata/cgroups/invalid.toml", imageRef, "/bin/sleep", "5"},
 			expectErrorCode: 255,
-			expectErrorOut:  "parsing error",
+			expectErrorOut:  "toml: expected character",
 			rootfull:        true,
 			rootless:        true,
 			skipOCI:         false,
@@ -375,18 +380,18 @@ func (c *ctx) actionApply(t *testing.T, profile e2e.Profile, imageRef string) {
 
 func (c *ctx) actionApplyRoot(t *testing.T) {
 	e2e.EnsureImage(t, c.env)
-	e2e.EnsureOCIImage(t, c.env)
+	e2e.EnsureOCIArchive(t, c.env)
 	t.Run(e2e.RootProfile.String(), func(t *testing.T) {
 		c.actionApply(t, e2e.RootProfile, c.env.ImagePath)
 	})
 	t.Run(e2e.OCIRootProfile.String(), func(t *testing.T) {
-		c.actionApply(t, e2e.OCIRootProfile, "oci-archive:"+c.env.OCIImagePath)
+		c.actionApply(t, e2e.OCIRootProfile, "oci-archive:"+c.env.OCIArchivePath)
 	})
 }
 
 func (c *ctx) actionApplyRootless(t *testing.T) {
 	e2e.EnsureImage(t, c.env)
-	e2e.EnsureOCIImage(t, c.env)
+	e2e.EnsureOCIArchive(t, c.env)
 	for _, profile := range []e2e.Profile{e2e.UserProfile, e2e.UserNamespaceProfile, e2e.FakerootProfile} {
 		t.Run(profile.String(), func(t *testing.T) {
 			c.actionApply(t, profile, c.env.ImagePath)
@@ -394,7 +399,7 @@ func (c *ctx) actionApplyRootless(t *testing.T) {
 	}
 	for _, profile := range []e2e.Profile{e2e.OCIUserProfile, e2e.OCIFakerootProfile} {
 		t.Run(profile.String(), func(t *testing.T) {
-			c.actionApply(t, profile, "oci-archive:"+c.env.OCIImagePath)
+			c.actionApply(t, profile, "oci-archive:"+c.env.OCIArchivePath)
 		})
 	}
 }
@@ -421,8 +426,11 @@ var resourceFlagTests = []resourceFlagTest{
 		args:            []string{"--blkio-weight", "50"},
 		expectErrorCode: 0,
 		controllerV1:    "blkio",
-		// This is the new path. Older kernels may have only `blkio.weight`
-		resourceV1:   "blkio.bfq.weight",
+		// Could be `blkio.bfq.weight` if bfq is available. However, under
+		// cgroups v1 older crun will not set blkio.bfq.weight, so only test
+		// with blkio.weight.
+		// Ref: https://github.com/containers/crun/issues/1157
+		resourceV1:   "blkio.weight",
 		expectV1:     "50",
 		delegationV2: "io",
 		resourceV2:   "io.bfq.weight",
@@ -614,18 +622,18 @@ func (c *ctx) actionFlagV2(t *testing.T, tt resourceFlagTest, profile e2e.Profil
 
 func (c *ctx) actionFlagsRoot(t *testing.T) {
 	e2e.EnsureImage(t, c.env)
-	e2e.EnsureOCIImage(t, c.env)
+	e2e.EnsureOCIArchive(t, c.env)
 	t.Run(e2e.RootProfile.String(), func(t *testing.T) {
 		c.actionFlags(t, e2e.RootProfile, c.env.ImagePath)
 	})
 	t.Run(e2e.OCIRootProfile.String(), func(t *testing.T) {
-		c.actionFlags(t, e2e.OCIRootProfile, "oci-archive:"+c.env.OCIImagePath)
+		c.actionFlags(t, e2e.OCIRootProfile, "oci-archive:"+c.env.OCIArchivePath)
 	})
 }
 
 func (c *ctx) actionFlagsRootless(t *testing.T) {
 	e2e.EnsureImage(t, c.env)
-	e2e.EnsureOCIImage(t, c.env)
+	e2e.EnsureOCIArchive(t, c.env)
 	for _, profile := range []e2e.Profile{e2e.UserProfile, e2e.UserNamespaceProfile, e2e.FakerootProfile} {
 		t.Run(profile.String(), func(t *testing.T) {
 			c.actionFlags(t, profile, c.env.ImagePath)
@@ -633,7 +641,7 @@ func (c *ctx) actionFlagsRootless(t *testing.T) {
 	}
 	for _, profile := range []e2e.Profile{e2e.OCIUserProfile, e2e.OCIFakerootProfile} {
 		t.Run(profile.String(), func(t *testing.T) {
-			c.actionFlags(t, profile, "oci-archive:"+c.env.OCIImagePath)
+			c.actionFlags(t, profile, "oci-archive:"+c.env.OCIArchivePath)
 		})
 	}
 }
