@@ -41,7 +41,11 @@ func (c ctx) testDockerPulls(t *testing.T) {
 	if err != nil {
 		t.Fatalf("failed to create temporary directory: %+v", err)
 	}
-	defer os.RemoveAll(tmpPath)
+	t.Cleanup(func() {
+		if !t.Failed() {
+			os.RemoveAll(tmpPath)
+		}
+	})
 
 	tmpImage := filepath.Join(tmpPath, tmpContainerFile)
 
@@ -118,17 +122,18 @@ func (c ctx) testDockerPulls(t *testing.T) {
 func (c ctx) testDockerHost(t *testing.T) {
 	require.Command(t, "docker")
 
+	// Temporary homedir for docker commands, so invoking docker doesn't create
+	// a ~/.docker that may interfere elsewhere.
+	tmpHome, cleanupHome := e2e.MakeTempDir(t, c.env.TestDir, "docker-", "")
+	t.Cleanup(func() { e2e.Privileged(cleanupHome)(t) })
+
 	// Create a Dockerfile for a small image we can build locally
-	tmpPath, err := fs.MakeTmpDir(c.env.TestDir, "docker-", 0o755)
-	err = errors.Wrapf(err, "creating temporary directory in %q for docker host test", c.env.TestDir)
-	if err != nil {
-		t.Fatalf("failed to create temporary directory: %+v", err)
-	}
-	defer os.RemoveAll(tmpPath)
+	tmpPath, cleanup := e2e.MakeTempDir(t, c.env.TestDir, "docker-", "")
+	t.Cleanup(func() { cleanup(t) })
 
 	dockerfile := filepath.Join(tmpPath, "Dockerfile")
 	dockerfileContent := []byte("FROM alpine:latest\n")
-	err = os.WriteFile(dockerfile, dockerfileContent, 0o644)
+	err := os.WriteFile(dockerfile, dockerfileContent, 0o644)
 	if err != nil {
 		t.Fatalf("failed to create temporary Dockerfile: %+v", err)
 	}
@@ -141,6 +146,7 @@ func (c ctx) testDockerHost(t *testing.T) {
 	e2e.Privileged(func(t *testing.T) {
 		cmd := exec.Command("docker", "build", "-t", dockerRef, tmpPath)
 		cmd.Dir = tmpPath
+		cmd.Env = append(cmd.Env, "HOME="+tmpHome)
 		out, err := cmd.CombinedOutput()
 		t.Log(cmd.Args)
 		if err != nil {
@@ -249,6 +255,7 @@ func (c ctx) testDockerHost(t *testing.T) {
 	// Clean up docker image
 	e2e.Privileged(func(t *testing.T) {
 		cmd := exec.Command("docker", "rmi", dockerRef)
+		cmd.Env = append(cmd.Env, "HOME="+tmpHome)
 		_, err = cmd.Output()
 		if err != nil {
 			t.Fatalf("Unexpected error while cleaning up docker image.\n%s", err)
@@ -259,7 +266,11 @@ func (c ctx) testDockerHost(t *testing.T) {
 // AUFS sanity tests
 func (c ctx) testDockerAUFS(t *testing.T) {
 	imageDir, cleanup := e2e.MakeTempDir(t, c.env.TestDir, "aufs-", "")
-	defer cleanup(t)
+	t.Cleanup(func() {
+		if !t.Failed() {
+			cleanup(t)
+		}
+	})
 	imagePath := filepath.Join(imageDir, "container")
 
 	c.env.RunSingularity(
@@ -306,7 +317,11 @@ func (c ctx) testDockerAUFS(t *testing.T) {
 // Check force permissions for user builds #977
 func (c ctx) testDockerPermissions(t *testing.T) {
 	imageDir, cleanup := e2e.MakeTempDir(t, c.env.TestDir, "perm-", "")
-	defer cleanup(t)
+	t.Cleanup(func() {
+		if !t.Failed() {
+			cleanup(t)
+		}
+	})
 	imagePath := filepath.Join(imageDir, "container")
 
 	c.env.RunSingularity(
@@ -352,7 +367,11 @@ func (c ctx) testDockerPermissions(t *testing.T) {
 // Check whiteout of symbolic links #1592 #1576
 func (c ctx) testDockerWhiteoutSymlink(t *testing.T) {
 	imageDir, cleanup := e2e.MakeTempDir(t, c.env.TestDir, "whiteout-", "")
-	defer cleanup(t)
+	t.Cleanup(func() {
+		if !t.Failed() {
+			cleanup(t)
+		}
+	})
 	imagePath := filepath.Join(imageDir, "container")
 
 	c.env.RunSingularity(
@@ -372,7 +391,11 @@ func (c ctx) testDockerWhiteoutSymlink(t *testing.T) {
 
 func (c ctx) testDockerDefFile(t *testing.T) {
 	imageDir, cleanup := e2e.MakeTempDir(t, c.env.TestDir, "def-", "")
-	defer cleanup(t)
+	t.Cleanup(func() {
+		if !t.Failed() {
+			cleanup(t)
+		}
+	})
 	imagePath := filepath.Join(imageDir, "container")
 
 	getKernelMajor := func(t *testing.T) (major int) {
@@ -416,7 +439,7 @@ func (c ctx) testDockerDefFile(t *testing.T) {
 	}
 
 	for _, tt := range tests {
-		deffile := e2e.PrepareDefFile(e2e.DefFileDetails{
+		defFile := e2e.PrepareDefFile(e2e.DefFileDetails{
 			Bootstrap: "docker",
 			From:      tt.from,
 		})
@@ -426,7 +449,7 @@ func (c ctx) testDockerDefFile(t *testing.T) {
 			e2e.AsSubtest(tt.name),
 			e2e.WithProfile(e2e.RootProfile),
 			e2e.WithCommand("build"),
-			e2e.WithArgs([]string{imagePath, deffile}...),
+			e2e.WithArgs([]string{imagePath, defFile}...),
 			e2e.PreRun(func(t *testing.T) {
 				require.Arch(t, tt.archRequired)
 				if getKernelMajor(t) < tt.kernelMajorRequired {
@@ -434,14 +457,16 @@ func (c ctx) testDockerDefFile(t *testing.T) {
 				}
 			}),
 			e2e.PostRun(func(t *testing.T) {
-				defer os.Remove(imagePath)
-				defer os.Remove(deffile)
-
 				if t.Failed() {
 					return
 				}
 
 				c.env.ImageVerify(t, imagePath)
+
+				if !t.Failed() {
+					os.Remove(imagePath)
+					os.Remove(defFile)
+				}
 			}),
 			e2e.ExpectExit(0),
 		)
@@ -450,7 +475,11 @@ func (c ctx) testDockerDefFile(t *testing.T) {
 
 func (c ctx) testDockerRegistry(t *testing.T) {
 	imageDir, cleanup := e2e.MakeTempDir(t, c.env.TestDir, "registry-", "")
-	defer cleanup(t)
+	t.Cleanup(func() {
+		if !t.Failed() {
+			cleanup(t)
+		}
+	})
 	imagePath := filepath.Join(imageDir, "container")
 
 	tests := []struct {
@@ -496,14 +525,16 @@ func (c ctx) testDockerRegistry(t *testing.T) {
 			e2e.WithCommand("build"),
 			e2e.WithArgs("--disable-cache", "--no-https", imagePath, defFile),
 			e2e.PostRun(func(t *testing.T) {
-				defer os.Remove(imagePath)
-				defer os.Remove(defFile)
-
 				if t.Failed() || tt.exit != 0 {
 					return
 				}
 
 				c.env.ImageVerify(t, imagePath)
+
+				if !t.Failed() {
+					os.Remove(imagePath)
+					os.Remove(defFile)
+				}
 			}),
 			e2e.ExpectExit(tt.exit),
 		)
@@ -512,7 +543,11 @@ func (c ctx) testDockerRegistry(t *testing.T) {
 
 func (c ctx) testDockerLabels(t *testing.T) {
 	imageDir, cleanup := e2e.MakeTempDir(t, c.env.TestDir, "labels-", "")
-	defer cleanup(t)
+	t.Cleanup(func() {
+		if !t.Failed() {
+			cleanup(t)
+		}
+	})
 	imagePath := filepath.Join(imageDir, "container")
 
 	// Test container & set labels
@@ -552,7 +587,11 @@ func (c ctx) testDockerLabels(t *testing.T) {
 //nolint:dupl
 func (c ctx) testDockerCMD(t *testing.T) {
 	imageDir, cleanup := e2e.MakeTempDir(t, c.env.TestDir, "docker-", "")
-	defer cleanup(t)
+	t.Cleanup(func() {
+		if !t.Failed() {
+			cleanup(t)
+		}
+	})
 	imagePath := filepath.Join(imageDir, "container")
 
 	home, err := os.UserHomeDir()
@@ -651,7 +690,11 @@ func (c ctx) testDockerCMD(t *testing.T) {
 //nolint:dupl
 func (c ctx) testDockerENTRYPOINT(t *testing.T) {
 	imageDir, cleanup := e2e.MakeTempDir(t, c.env.TestDir, "docker-", "")
-	defer cleanup(t)
+	t.Cleanup(func() {
+		if !t.Failed() {
+			cleanup(t)
+		}
+	})
 	imagePath := filepath.Join(imageDir, "container")
 
 	home, err := os.UserHomeDir()
@@ -737,7 +780,11 @@ func (c ctx) testDockerENTRYPOINT(t *testing.T) {
 //nolint:dupl
 func (c ctx) testDockerCMDENTRYPOINT(t *testing.T) {
 	imageDir, cleanup := e2e.MakeTempDir(t, c.env.TestDir, "docker-", "")
-	defer cleanup(t)
+	t.Cleanup(func() {
+		if !t.Failed() {
+			cleanup(t)
+		}
+	})
 	imagePath := filepath.Join(imageDir, "container")
 
 	home, err := os.UserHomeDir()
@@ -836,35 +883,86 @@ func (c ctx) testDockerCMDQuotes(t *testing.T) {
 
 // Check that the USER in a docker container is honored under --oci mode
 func (c ctx) testDockerUSER(t *testing.T) {
+	dockerURI := "docker://sylabsio/docker-user"
 	tests := []struct {
-		name         string
-		expectOutput string
-		profile      e2e.Profile
+		name          string
+		cmd           string
+		args          []string
+		expectOutputs []e2e.SingularityCmdResultOp
+		profile       e2e.Profile
+		expectExit    int
 	}{
 		// Sanity check singularity native engine... no support for USER
 		{
 			name:    "default",
+			cmd:     "run",
 			profile: e2e.UserProfile,
-			expectOutput: fmt.Sprintf("uid=%d(%s) gid=%d",
-				e2e.UserProfile.ContainerUser(t).UID,
-				e2e.UserProfile.ContainerUser(t).Name,
-				e2e.UserProfile.ContainerUser(t).GID),
+			args:    []string{dockerURI},
+			expectOutputs: []e2e.SingularityCmdResultOp{
+				e2e.ExpectOutput(e2e.ContainMatch, fmt.Sprintf("uid=%d(%s) gid=%d",
+					e2e.UserProfile.ContainerUser(t).UID,
+					e2e.UserProfile.ContainerUser(t).Name,
+					e2e.UserProfile.ContainerUser(t).GID,
+				)),
+			},
 		},
 		// `--oci` modes (USER honored by default)
 		{
-			name:         "OCIUser",
-			profile:      e2e.OCIUserProfile,
-			expectOutput: `uid=2000(testuser) gid=2000(testgroup)`,
+			name:    "OCIUser",
+			cmd:     "run",
+			profile: e2e.OCIUserProfile,
+			args:    []string{dockerURI},
+			expectOutputs: []e2e.SingularityCmdResultOp{
+				e2e.ExpectOutput(e2e.ContainMatch, `uid=2000(testuser) gid=2000(testgroup)`),
+			},
 		},
 		{
-			name:         "OCIFakeroot",
-			profile:      e2e.OCIFakerootProfile,
-			expectOutput: `uid=0(root) gid=0(root)`,
+			name:    "OCIFakeroot",
+			profile: e2e.OCIFakerootProfile,
+			args:    []string{dockerURI},
+			expectOutputs: []e2e.SingularityCmdResultOp{
+				e2e.ExpectOutput(e2e.ContainMatch, `uid=0(root) gid=0(root)`),
+			},
 		},
 		{
-			name:         "OCIRoot",
-			profile:      e2e.OCIRootProfile,
-			expectOutput: `uid=2000(testuser) gid=2000(testgroup)`,
+			name:    "OCIRoot",
+			cmd:     "run",
+			profile: e2e.OCIRootProfile,
+			args:    []string{dockerURI},
+			expectOutputs: []e2e.SingularityCmdResultOp{
+				e2e.ExpectOutput(e2e.ContainMatch, `uid=2000(testuser) gid=2000(testgroup)`),
+			},
+		},
+		// `--oci` modes: check that we don't override container-user's home directory
+		{
+			name:    "OrigHomeOCIUser",
+			cmd:     "exec",
+			profile: e2e.OCIUserProfile,
+			args:    []string{dockerURI, "env"},
+			expectOutputs: []e2e.SingularityCmdResultOp{
+				e2e.ExpectOutput(e2e.RegexMatch, `\bHOME=/home/testuser\b`),
+			},
+			expectExit: 0,
+		},
+		{
+			name:    "OrigHomeOCIFakeroot",
+			cmd:     "exec",
+			profile: e2e.OCIFakerootProfile,
+			args:    []string{dockerURI, "env"},
+			expectOutputs: []e2e.SingularityCmdResultOp{
+				e2e.ExpectOutput(e2e.RegexMatch, `\bHOME=/root\b`),
+			},
+			expectExit: 0,
+		},
+		{
+			name:    "OrigHomeOCIRoot",
+			cmd:     "exec",
+			profile: e2e.OCIRootProfile,
+			args:    []string{dockerURI, "env"},
+			expectOutputs: []e2e.SingularityCmdResultOp{
+				e2e.ExpectOutput(e2e.RegexMatch, `\bHOME=/home/testuser\b`),
+			},
+			expectExit: 0,
 		},
 	}
 
@@ -874,10 +972,8 @@ func (c ctx) testDockerUSER(t *testing.T) {
 			e2e.AsSubtest(tt.name),
 			e2e.WithProfile(tt.profile),
 			e2e.WithCommand("run"),
-			e2e.WithArgs("docker://sylabsio/docker-user"),
-			e2e.ExpectExit(0,
-				e2e.ExpectOutput(e2e.ContainMatch, tt.expectOutput),
-			),
+			e2e.WithArgs(tt.args...),
+			e2e.ExpectExit(tt.expectExit, tt.expectOutputs...),
 		)
 	}
 }
@@ -907,6 +1003,7 @@ func E2ETests(env e2e.TestEnv) testhelper.Tests {
 			// Regressions
 			t.Run("issue 4524", c.issue4524)
 			t.Run("issue 1286", c.issue1286)
+			t.Run("issue 1528", c.issue1528)
 		},
 		// Tests that are especially slow, or run against a local docker
 		// registry, can be run in parallel, with `--disable-cache` used within
@@ -916,6 +1013,7 @@ func E2ETests(env e2e.TestEnv) testhelper.Tests {
 		// Regressions
 		"issue 4943": c.issue4943,
 		"issue 5172": c.issue5172,
-		"issue 274":  c.issue274, // https://github.com/sylabs/singularity/issues/274
+		"issue 274":  c.issue274,  // https://github.com/sylabs/singularity/issues/274
+		"issue 1704": c.issue1704, // https://github.com/sylabs/singularity/issues/1704
 	}
 }
