@@ -11,7 +11,6 @@ import (
 	"fmt"
 	"log"
 	"os"
-	"os/exec"
 	"path"
 	"path/filepath"
 	"strings"
@@ -58,22 +57,26 @@ func (c imgBuildTests) buildFrom(t *testing.T) {
 	// `singularity build -s /tmp/sand/ <URI>` works,
 	// see https://github.com/sylabs/singularity/issues/4407
 	tt := []struct {
-		name        string
-		dependency  string
-		buildSpec   string
-		requireArch string
+		name         string
+		dependency   string
+		buildSpec    string
+		requirements func(t *testing.T)
 	}{
 		// Disabled due to frequent download failures of the busybox tgz
 		// {
 		// 	name:      "BusyBox",
 		// 	buildSpec: "../examples/busybox/Singularity",
 		// 	// TODO: example has arch hard coded in download URL
-		// 	requireArch: "amd64",
+		//  requirements: func(t *testing.T) {
+		//   require.Arch(t, "amd64")
+		//  },
 		// },
 		{
-			name:       "Debootstrap",
-			dependency: "debootstrap",
-			buildSpec:  "../examples/debian/Singularity",
+			name:      "Debootstrap",
+			buildSpec: "../examples/debian/Singularity",
+			requirements: func(t *testing.T) {
+				require.Command(t, "debootstrap")
+			},
 		},
 		// TODO(mem): reenable this; disabled while shub is down
 		// {
@@ -98,28 +101,83 @@ func (c imgBuildTests) buildFrom(t *testing.T) {
 			buildSpec: c.env.OrasTestImage,
 		},
 		{
-			name:        "Yum",
-			dependency:  "yum",
-			buildSpec:   "../examples/centos/Singularity",
-			requireArch: "amd64",
+			name:      "Yum CentOS7",
+			buildSpec: "../examples/centos/Singularity",
+			requirements: func(t *testing.T) {
+				require.Command(t, "yum")
+				require.RPMMacro(t, "_db_backend", "bdb")
+				require.RPMMacro(t, "_dbpath", "/var/lib/rpm")
+				require.Arch(t, "amd64")
+			},
 		},
 		{
-			name:        "YumArm64",
-			dependency:  "yum",
-			buildSpec:   "../examples/centos-arm64/Singularity",
-			requireArch: "arm64",
+			name:       "YumArm64 CentOS 7",
+			dependency: "yum",
+			buildSpec:  "../examples/centos-arm64/Singularity",
+			requirements: func(t *testing.T) {
+				require.Command(t, "yum")
+				require.RPMMacro(t, "_db_backend", "bdb")
+				require.RPMMacro(t, "_dbpath", "/var/lib/rpm")
+				require.Arch(t, "arm64")
+			},
 		},
 		{
-			name:        "Zypper",
-			dependency:  "zypper",
-			buildSpec:   "../examples/opensuse/Singularity",
-			requireArch: "amd64",
+			name:      "Dnf AlmaLinux 9",
+			buildSpec: "../examples/almalinux/Singularity",
+			requirements: func(t *testing.T) {
+				require.Command(t, "dnf")
+				require.RPMMacro(t, "_db_backend", "sqlite")
+				require.RPMMacro(t, "_dbpath", "/var/lib/rpm")
+				require.Arch(t, "amd64")
+			},
 		},
 		{
-			name:        "ZypperArm64",
-			dependency:  "zypper",
-			buildSpec:   "../examples/opensuse-arm64/Singularity",
-			requireArch: "arm64",
+			name:       "DnfArm64 AlmaLinux 9",
+			dependency: "yum",
+			buildSpec:  "../examples/almalinux-arm64/Singularity",
+			requirements: func(t *testing.T) {
+				require.Command(t, "dnf")
+				require.RPMMacro(t, "_db_backend", "sqlite")
+				require.RPMMacro(t, "_dbpath", "/var/lib/rpm")
+				require.Arch(t, "arm64")
+			},
+		},
+		{
+			name:      "Dnf Fedora 37",
+			buildSpec: "../examples/fedora/Singularity",
+			requirements: func(t *testing.T) {
+				require.Command(t, "dnf")
+				require.RPMMacro(t, "_db_backend", "sqlite")
+				require.RPMMacro(t, "_dbpath", "/usr/lib/sysimage/rpm")
+				require.Arch(t, "amd64")
+			},
+		},
+		{
+			name:       "DnfArm64 Fedora 37",
+			dependency: "yum",
+			buildSpec:  "../examples/fedora-arm64/Singularity",
+			requirements: func(t *testing.T) {
+				require.Command(t, "dnf")
+				require.RPMMacro(t, "_db_backend", "sqlite")
+				require.RPMMacro(t, "_dbpath", "/usr/lib/sysimage/rpm")
+				require.Arch(t, "arm64")
+			},
+		},
+		{
+			name:      "Zypper",
+			buildSpec: "../examples/opensuse/Singularity",
+			requirements: func(t *testing.T) {
+				require.Command(t, "zypper")
+				require.Arch(t, "amd64")
+			},
+		},
+		{
+			name:      "ZypperArm64",
+			buildSpec: "../examples/opensuse-arm64/Singularity",
+			requirements: func(t *testing.T) {
+				require.Command(t, "zypper")
+				require.Arch(t, "arm64")
+			},
 		},
 	}
 
@@ -129,8 +187,13 @@ func (c imgBuildTests) buildFrom(t *testing.T) {
 
 		t.Run(profile.String(), func(t *testing.T) {
 			for _, tc := range tt {
+
 				dn, cleanup := c.tempDir(t, "build-from")
-				defer cleanup()
+				t.Cleanup(func() {
+					if !t.Failed() {
+						cleanup()
+					}
+				})
 
 				imagePath := path.Join(dn, "sandbox")
 
@@ -144,23 +207,17 @@ func (c imgBuildTests) buildFrom(t *testing.T) {
 					e2e.WithProfile(profile),
 					e2e.WithCommand("build"),
 					e2e.WithArgs(args...),
-					e2e.PreRun(func(t *testing.T) {
-						require.Arch(t, tc.requireArch)
-
-						if tc.dependency == "" {
-							return
-						}
-
-						if _, err := exec.LookPath(tc.dependency); err != nil {
-							t.Skipf("%v not found in path", tc.dependency)
-						}
-					}),
+					e2e.PreRun(tc.requirements),
 					e2e.PostRun(func(t *testing.T) {
 						if t.Failed() {
 							return
 						}
 
-						defer os.RemoveAll(imagePath)
+						t.Cleanup(func() {
+							if !t.Failed() {
+								os.RemoveAll(imagePath)
+							}
+						})
 						c.env.ImageVerify(t, imagePath)
 					}),
 					e2e.ExpectExit(0),
@@ -205,7 +262,11 @@ func (c imgBuildTests) nonRootBuild(t *testing.T) {
 
 	for _, tc := range tt {
 		dn, cleanup := c.tempDir(t, "non-root-build")
-		defer cleanup()
+		t.Cleanup(func() {
+			if !t.Failed() {
+				cleanup()
+			}
+		})
 
 		imagePath := path.Join(dn, "container")
 
@@ -236,13 +297,21 @@ func (c imgBuildTests) buildLocalImage(t *testing.T) {
 
 	tmpdir, cleanup := c.tempDir(t, "build-local-image")
 
-	defer cleanup()
+	t.Cleanup(func() {
+		if !t.Failed() {
+			cleanup()
+		}
+	})
 
 	liDefFile := e2e.PrepareDefFile(e2e.DefFileDetails{
 		Bootstrap: "localimage",
 		From:      c.env.ImagePath,
 	})
-	defer os.Remove(liDefFile)
+	t.Cleanup(func() {
+		if !t.Failed() {
+			os.Remove(liDefFile)
+		}
+	})
 
 	labels := make(map[string]string)
 	labels["FOO"] = "bar"
@@ -251,7 +320,11 @@ func (c imgBuildTests) buildLocalImage(t *testing.T) {
 		From:      c.env.ImagePath,
 		Labels:    labels,
 	})
-	defer os.Remove(liLabelDefFile)
+	t.Cleanup(func() {
+		if !t.Failed() {
+			os.Remove(liLabelDefFile)
+		}
+	})
 
 	sandboxImage := path.Join(tmpdir, "test-sandbox")
 
@@ -271,7 +344,11 @@ func (c imgBuildTests) buildLocalImage(t *testing.T) {
 		From:      sandboxImage,
 		Labels:    labels,
 	})
-	defer os.Remove(localSandboxDefFile)
+	t.Cleanup(func() {
+		if !t.Failed() {
+			os.Remove(localSandboxDefFile)
+		}
+	})
 
 	tt := []struct {
 		name      string
@@ -301,7 +378,11 @@ func (c imgBuildTests) buildLocalImage(t *testing.T) {
 						if t.Failed() {
 							return
 						}
-						defer os.RemoveAll(imagePath)
+						t.Cleanup(func() {
+							if !t.Failed() {
+								os.RemoveAll(imagePath)
+							}
+						})
 						c.env.ImageVerify(t, imagePath)
 					}),
 					e2e.ExpectExit(0),
@@ -313,7 +394,11 @@ func (c imgBuildTests) buildLocalImage(t *testing.T) {
 
 func (c imgBuildTests) badPath(t *testing.T) {
 	dn, cleanup := c.tempDir(t, "bad-path")
-	defer cleanup()
+	t.Cleanup(func() {
+		if !t.Failed() {
+			cleanup()
+		}
+	})
 
 	imagePath := path.Join(dn, "container")
 
@@ -332,7 +417,11 @@ func (c imgBuildTests) buildMultiStageDefinition(t *testing.T) {
 	if err != nil {
 		log.Fatal(err)
 	}
-	defer os.Remove(tmpfile) // clean up
+	t.Cleanup(func() {
+		if !t.Failed() {
+			os.Remove(tmpfile)
+		}
+	})
 
 	tests := []struct {
 		name    string
@@ -511,7 +600,11 @@ func (c imgBuildTests) buildMultiStageDefinition(t *testing.T) {
 
 	for _, tt := range tests {
 		dn, cleanup := c.tempDir(t, "multi-stage-definition")
-		defer cleanup()
+		t.Cleanup(func() {
+			if !t.Failed() {
+				cleanup()
+			}
+		})
 
 		imagePath := path.Join(dn, "container")
 
@@ -526,7 +619,11 @@ func (c imgBuildTests) buildMultiStageDefinition(t *testing.T) {
 			e2e.WithCommand("build"),
 			e2e.WithArgs(args...),
 			e2e.PostRun(func(t *testing.T) {
-				defer os.Remove(defFile)
+				t.Cleanup(func() {
+					if !t.Failed() {
+						os.Remove(defFile)
+					}
+				})
 
 				e2e.DefinitionImageVerify(t, c.env.CmdPath, imagePath, tt.correct)
 			}),
@@ -542,7 +639,11 @@ func (c imgBuildTests) buildDefinition(t *testing.T) {
 	if err != nil {
 		log.Fatal(err)
 	}
-	defer os.Remove(tmpfile) // clean up
+	t.Cleanup(func() {
+		if !t.Failed() {
+			os.Remove(tmpfile)
+		}
+	})
 
 	tt := map[string]e2e.DefFileDetails{
 		"Empty": {
@@ -807,7 +908,11 @@ func (c imgBuildTests) buildDefinition(t *testing.T) {
 		t.Run(profile.String(), func(t *testing.T) {
 			for name, dfd := range tt {
 				dn, cleanup := c.tempDir(t, "build-definition")
-				defer cleanup()
+				t.Cleanup(func() {
+					if !t.Failed() {
+						cleanup()
+					}
+				})
 
 				imagePath := path.Join(dn, "container")
 
@@ -823,7 +928,11 @@ func (c imgBuildTests) buildDefinition(t *testing.T) {
 						if t.Failed() {
 							return
 						}
-						defer os.Remove(defFile)
+						t.Cleanup(func() {
+							if !t.Failed() {
+								os.Remove(defFile)
+							}
+						})
 						e2e.DefinitionImageVerify(t, c.env.CmdPath, imagePath, dfd)
 					}),
 					e2e.ExpectExit(0),
@@ -858,7 +967,11 @@ func (c imgBuildTests) buildEncryptPemFile(t *testing.T) {
 	// We create a temporary directory to store the image, making sure tests
 	// will not pollute each other
 	dn, cleanup := c.tempDir(t, "pem-encryption")
-	defer cleanup()
+	t.Cleanup(func() {
+		if !t.Failed() {
+			cleanup()
+		}
+	})
 
 	// Generate the PEM file
 	pemFile, _ := e2e.GeneratePemFiles(t, c.env.TestDir)
@@ -926,7 +1039,11 @@ func (c imgBuildTests) buildEncryptPassphrase(t *testing.T) {
 	// We create a temporary directory to store the image, making sure tests
 	// will not pollute each other
 	dn, cleanup := c.tempDir(t, "passphrase-encryption")
-	defer cleanup()
+	t.Cleanup(func() {
+		if !t.Failed() {
+			cleanup()
+		}
+	})
 
 	// If the version of cryptsetup is not compatible with Singularity encryption,
 	// the build commands are expected to fail
@@ -1024,7 +1141,11 @@ func (c imgBuildTests) buildUpdateSandbox(t *testing.T) {
 	const badSandbox = "/bad/sandbox/path"
 
 	testDir, cleanup := e2e.MakeTempDir(t, c.env.TestDir, "build-sandbox-", "")
-	defer e2e.Privileged(cleanup)
+	t.Cleanup(func() {
+		if !t.Failed() {
+			e2e.Privileged(cleanup)
+		}
+	})
 
 	tests := []struct {
 		name     string
@@ -1073,10 +1194,10 @@ func (c imgBuildTests) buildUpdateSandbox(t *testing.T) {
 // buildWithFingerprint checks that we correctly verify a source image fingerprint when specified
 func (c imgBuildTests) buildWithFingerprint(t *testing.T) {
 	tmpDir, remove := e2e.MakeTempDir(t, "", "imgbuild-fingerprint-", "")
-	defer func() {
+	t.Cleanup(func() {
 		c.env.KeyringDir = ""
 		remove(t)
-	}()
+	})
 
 	pgpDir, _ := e2e.MakeSyPGPDir(t, tmpDir)
 	c.env.KeyringDir = pgpDir
@@ -1240,7 +1361,11 @@ func (c imgBuildTests) buildWithFingerprint(t *testing.T) {
 		if err != nil {
 			log.Fatal(err)
 		}
-		defer os.Remove(defFile)
+		t.Cleanup(func() {
+			if !t.Failed() {
+				os.Remove(defFile)
+			}
+		})
 		c.env.RunSingularity(t,
 			e2e.AsSubtest(tt.name),
 			e2e.WithProfile(e2e.RootProfile),
@@ -1258,7 +1383,11 @@ func (c imgBuildTests) buildBindMount(t *testing.T) {
 	e2e.EnsureImage(t, c.env)
 
 	tmpdir, cleanup := c.tempDir(t, "build-local-image")
-	defer cleanup()
+	t.Cleanup(func() {
+		if !t.Failed() {
+			cleanup()
+		}
+	})
 
 	dir, _ := e2e.MakeTempDir(t, tmpdir, "mount", "")
 
@@ -1426,7 +1555,11 @@ func (c imgBuildTests) testWritableTmpfs(t *testing.T) {
 	e2e.EnsureImage(t, c.env)
 
 	tmpdir, cleanup := c.tempDir(t, "build-writabletmpfs-test")
-	defer cleanup()
+	t.Cleanup(func() {
+		if !t.Failed() {
+			cleanup()
+		}
+	})
 
 	// Definition will attempt to touch a file in /var/test during %test.
 	// This would fail without a writable tmpfs.
@@ -1450,14 +1583,18 @@ func (c imgBuildTests) buildLibraryHost(t *testing.T) {
 	e2e.EnsureImage(t, c.env)
 
 	tmpdir, cleanup := c.tempDir(t, "build-libraryhost-test")
-	defer cleanup()
+	t.Cleanup(func() {
+		if !t.Failed() {
+			cleanup()
+		}
+	})
 
 	// Library hostname in the From URI
 	// The hostname is invalid, and we should get an error to that effect.
 	definition := "Bootstrap: library\nFrom: library.example.com/test/test/test:latest\n"
 
 	defFile := e2e.RawDefFile(t, tmpdir, strings.NewReader(definition))
-	imagePath := filepath.Join(tmpdir, "image-libaryhost")
+	imagePath := filepath.Join(tmpdir, "image-libraryhost")
 	c.env.RunSingularity(
 		t,
 		e2e.WithProfile(e2e.RootProfile),
@@ -1467,7 +1604,7 @@ func (c imgBuildTests) buildLibraryHost(t *testing.T) {
 			os.Remove(defFile)
 		}),
 		e2e.ExpectExit(255,
-			e2e.ExpectError(e2e.ContainMatch, "dial tcp: lookup library.example.com: no such host"),
+			e2e.ExpectError(e2e.ContainMatch, "no such host"),
 		),
 	)
 }
@@ -1494,6 +1631,11 @@ func (c imgBuildTests) buildProot(t *testing.T) {
 			name:      "Ubuntu",
 			buildSpec: "testdata/proot_ubuntu.def",
 		},
+		// See: https://github.com/sylabs/singularity/issues/1643
+		{
+			name:      "MultiStage",
+			buildSpec: "testdata/proot_multistage.def",
+		},
 	}
 
 	profiles := []e2e.Profile{e2e.UserProfile}
@@ -1503,7 +1645,11 @@ func (c imgBuildTests) buildProot(t *testing.T) {
 		t.Run(profile.String(), func(t *testing.T) {
 			for _, tc := range tt {
 				dn, cleanup := c.tempDir(t, "build-proot")
-				defer cleanup()
+				t.Cleanup(func() {
+					if !t.Failed() {
+						cleanup()
+					}
+				})
 
 				imagePath := path.Join(dn, "image.sif")
 
@@ -1522,7 +1668,11 @@ func (c imgBuildTests) buildProot(t *testing.T) {
 							return
 						}
 
-						defer os.RemoveAll(imagePath)
+						t.Cleanup(func() {
+							if !t.Failed() {
+								os.RemoveAll(imagePath)
+							}
+						})
 						c.env.ImageVerify(t, imagePath)
 					}),
 					e2e.ExpectExit(0),
@@ -1535,7 +1685,11 @@ func (c imgBuildTests) buildProot(t *testing.T) {
 // Check that test and runscript that specify a custom #! use it as the interpreter.
 func (c imgBuildTests) buildCustomShebang(t *testing.T) {
 	tmpdir, cleanup := c.tempDir(t, "build-shebang-test")
-	defer cleanup()
+	t.Cleanup(func() {
+		if !t.Failed() {
+			cleanup()
+		}
+	})
 
 	definition := `Bootstrap: localimage
 From: %s
@@ -1588,6 +1742,45 @@ cat /proc/$$/cmdline`
 	)
 }
 
+// actionNoSetgoups checks that supplementary groups are visible, mapped to
+// nobody, in the container with --fakeroot --no-setgroups.
+func (c imgBuildTests) buildNoSetgroups(t *testing.T) {
+	tmpdir, cleanup := c.tempDir(t, "build-nosetgroups-test")
+	t.Cleanup(func() {
+		if !t.Failed() {
+			cleanup()
+		}
+	})
+
+	definition := `Bootstrap: localimage
+From: %s
+
+%%post
+    groups`
+
+	definition = fmt.Sprintf(definition, e2e.BusyboxSIF(t))
+
+	defFile := e2e.RawDefFile(t, tmpdir, strings.NewReader(definition))
+	imagePath := filepath.Join(tmpdir, "image-nosetgroups")
+
+	// Inside the e2e-tests we will be a member of our user group + single supplementary group.
+	// With `--fakeroot --no-setgroups` we should see these map to:
+	//    root nobody
+	c.env.RunSingularity(
+		t,
+		e2e.WithProfile(e2e.FakerootProfile),
+		e2e.WithCommand("build"),
+		e2e.WithArgs("--no-setgroups", "-F", imagePath, defFile),
+		e2e.PostRun(func(t *testing.T) {
+			os.Remove(defFile)
+			os.Remove(imagePath)
+		}),
+		e2e.ExpectExit(0,
+			e2e.ExpectOutput(e2e.ExactMatch, "root nobody"),
+		),
+	)
+}
+
 // E2ETests is the main func to trigger the test suite
 func E2ETests(env e2e.TestEnv) testhelper.Tests {
 	c := imgBuildTests{
@@ -1610,6 +1803,7 @@ func E2ETests(env e2e.TestEnv) testhelper.Tests {
 		"library host":                    c.buildLibraryHost,          // build image with hostname in library URI
 		"proot":                           c.buildProot,                // build image as an unpriv user with proot
 		"customShebang":                   c.buildCustomShebang,        // build image with custom #! in %test and %runscript
+		"no-setgroups":                    c.buildNoSetgroups,          // build with --fakeroot --no-setgroups
 		"issue 3848":                      c.issue3848,                 // https://github.com/hpcng/singularity/issues/3848
 		"issue 4203":                      c.issue4203,                 // https://github.com/sylabs/singularity/issues/4203
 		"issue 4407":                      c.issue4407,                 // https://github.com/sylabs/singularity/issues/4407
