@@ -1,5 +1,5 @@
 // Copyright (c) 2020, Control Command Inc. All rights reserved.
-// Copyright (c) 2020-2022, Sylabs Inc. All rights reserved.
+// Copyright (c) 2020-2023, Sylabs Inc. All rights reserved.
 // This software is licensed under a 3-clause BSD license. Please consult the LICENSE.md file
 // distributed with the sources of this project regarding your rights to use or distribute this
 // software.
@@ -175,7 +175,9 @@ func verifyCertificate(c *x509.Certificate, intermediates, roots *x509.CertPool)
 
 // getOpts returns integrity.VerifierOpt necessary to validate f.
 func (v verifier) getOpts(ctx context.Context, f *sif.FileImage) ([]integrity.VerifierOpt, error) {
-	var iopts []integrity.VerifierOpt
+	iopts := []integrity.VerifierOpt{
+		integrity.OptVerifyWithContext(ctx),
+	}
 
 	// Add key material from certificate(s).
 	for _, c := range v.certs {
