@@ -1,5 +1,5 @@
 // Copyright (c) 2020, Control Command Inc. All rights reserved.
-// Copyright (c) 2018-2022, Sylabs Inc. All rights reserved.
+// Copyright (c) 2018-2023, Sylabs Inc. All rights reserved.
 // This software is licensed under a 3-clause BSD license. Please consult the
 // LICENSE.md file distributed with the sources of this project regarding your
 // rights to use or distribute this software.
@@ -8,6 +8,7 @@ package singularity
 
 import (
 	"bufio"
+	"context"
 	"encoding/json"
 	"errors"
 	"fmt"
@@ -597,6 +598,7 @@ func (e *EngineOperations) prepareContainerConfig(starterConfig *starter.Config)
 
 		starterConfig.SetHybridWorkflow(true)
 		starterConfig.SetAllowSetgroups(true)
+		starterConfig.SetNoSetgroups(e.EngineConfig.GetNoSetgroups())
 
 		starterConfig.SetTargetUID(0)
 		starterConfig.SetTargetGID([]int{0})
@@ -1218,7 +1220,7 @@ func (e *EngineOperations) loadImages(starterConfig *starter.Config) error {
 				}
 			}
 
-			if ok, err := ecl.ShouldRunFp(img.File, kr); err != nil {
+			if ok, err := ecl.ShouldRunFp(context.TODO(), img.File, kr); err != nil {
 				return fmt.Errorf("while checking container image with ECL: %s", err)
 			} else if !ok {
 				return errors.New("image prohibited by ECL")
