@@ -130,13 +130,6 @@ func (l *Launcher) Exec(ctx context.Context, image string, process string, args
 	// Overlay or writable image requested?
 	l.engineConfig.SetOverlayImage(l.cfg.OverlayPaths)
 	l.engineConfig.SetWritableImage(l.cfg.Writable)
-	// --writable-tmpfs is for an ephemeral overlay, doesn't make sense if also asking to write to image itself.
-	if l.cfg.Writable && l.cfg.WritableTmpfs {
-		sylog.Warningf("Disabling --writable-tmpfs flag, mutually exclusive with --writable")
-		l.engineConfig.SetWritableTmpfs(false)
-	} else {
-		l.engineConfig.SetWritableTmpfs(l.cfg.WritableTmpfs)
-	}
 
 	// Check key is available for encrypted image, if applicable.
 	// If we are joining an instance, then any encrypted image is already mounted.
@@ -181,12 +174,21 @@ func (l *Launcher) Exec(ctx context.Context, image string, process string, args
 	l.setNoMountFlags()
 
 	// GPU configuration may add library bind to /.singularity.d/libs.
+	// Note: --nvccli may implicitly add --writable-tmpfs, so handle that *after* GPUs.
 	if err := l.SetGPUConfig(); err != nil {
 		// We must fatal on error, as we are checking for correct ownership of nvidia-container-cli,
 		// which is important to maintain security.
 		sylog.Fatalf("While setting GPU configuration: %s", err)
 	}
 
+	// --writable-tmpfs is for an ephemeral overlay, doesn't make sense if also asking to write to image itself.
+	if l.cfg.Writable && l.cfg.WritableTmpfs {
+		sylog.Warningf("Disabling --writable-tmpfs flag, mutually exclusive with --writable")
+		l.engineConfig.SetWritableTmpfs(false)
+	} else {
+		l.engineConfig.SetWritableTmpfs(l.cfg.WritableTmpfs)
+	}
+
 	// If proot is requested (we are running an unprivileged build, without userns) we must bind it
 	// into the container /.singularity.d/libs.
 	if l.cfg.Proot != "" && l.uid != 0 {
@@ -208,7 +210,11 @@ func (l *Launcher) Exec(ctx context.Context, image string, process string, args
 
 	// If user wants to set a hostname, it requires the UTS namespace.
 	if l.cfg.Hostname != "" {
-		l.cfg.Namespaces.UTS = true
+		// This is a sanity-check; actionPreRun in actions.go should have prevented this scenario from arising.
+		if !l.cfg.Namespaces.UTS {
+			return fmt.Errorf("internal error: trying to set hostname without UTS namespace")
+		}
+
 		l.engineConfig.SetHostname(l.cfg.Hostname)
 	}
 
@@ -254,6 +260,14 @@ func (l *Launcher) Exec(ctx context.Context, image string, process string, args
 	if l.cfg.Fakeroot {
 		l.cfg.Namespaces.User = true
 	}
+	// Allow optional skipping of setgroups in --fakeroot mode.
+	if l.cfg.NoSetgroups {
+		if l.cfg.Fakeroot {
+			l.engineConfig.SetNoSetgroups(l.cfg.NoSetgroups)
+		} else {
+			sylog.Warningf("--no-setgroups only applies to --fakeroot mode")
+		}
+	}
 
 	l.setCgroups(instanceName)
 
@@ -293,7 +307,7 @@ func (l *Launcher) Exec(ctx context.Context, image string, process string, args
 		if l.cfg.Boot {
 			l.cfg.Namespaces.UTS = true
 			l.cfg.Namespaces.Net = true
-			if l.cfg.Hostname == "" {
+			if len(l.cfg.Hostname) < 1 {
 				l.engineConfig.SetHostname(instanceName)
 			}
 			if !l.cfg.KeepPrivs {
@@ -650,8 +664,8 @@ func (l *Launcher) setHome() error {
 
 	// Handle any user request to override the home directory source/dest
 	homeSlice := strings.Split(l.cfg.HomeDir, ":")
-	if len(homeSlice) > 2 || len(homeSlice) == 0 {
-		return fmt.Errorf("home argument has incorrect number of elements: %v", len(homeSlice))
+	if len(homeSlice) < 1 || len(homeSlice) > 2 {
+		return fmt.Errorf("home argument has incorrect number of elements: %v", homeSlice)
 	}
 	l.engineConfig.SetHomeSource(homeSlice[0])
 	if len(homeSlice) == 1 {
@@ -800,8 +814,6 @@ func (l *Launcher) setNamespaces() {
 	// so we fallback to none
 	if l.cfg.Namespaces.Net {
 		if l.cfg.Fakeroot && l.cfg.Network != "none" {
-			l.engineConfig.SetNetwork("fakeroot")
-
 			if buildcfg.SINGULARITY_SUID_INSTALL == 0 || !l.engineConfig.File.AllowSetuid {
 				sylog.Warningf(
 					"fakeroot with unprivileged installation or 'allow setuid = no' " +
