@@ -1,4 +1,6 @@
-// Copyright (c) 2018-2021, Sylabs Inc. All rights reserved.
+// Copyright (c) 2018-2023, Sylabs Inc. All rights reserved.
+// Copyright (c) Contributors to the Apptainer project, established as
+//   Apptainer a Series of LF Projects LLC.
 // This software is licensed under a 3-clause BSD license. Please consult the
 // LICENSE.md file distributed with the sources of this project regarding your
 // rights to use or distribute this software.
@@ -141,11 +143,9 @@ var authorizedTags = map[AuthorizedTag]struct {
 	FinalTag:     {true, 16},
 }
 
-var authorizedImage = map[string]fsContext{
-	"encryptfs": {true},
-	"ext3":      {true},
-	"squashfs":  {true},
-}
+// authorizedImage holds fs types that are authorized for use in loopback image mounts.
+// Nothing is allowed by default. Use AuthorizeImageFS to add.
+var authorizedImage = map[string]fsContext{}
 
 var authorizedFS = map[string]fsContext{
 	"overlay": {true},
@@ -615,7 +615,7 @@ func (p *Points) AddImage(tag AuthorizedTag, source string, dest string, fstype
 		return fmt.Errorf("ms_bind, ms_rec or ms_remount are not valid flags for image mount points")
 	}
 	if _, ok := authorizedImage[fstype]; !ok {
-		return fmt.Errorf("mount %s image is not authorized", fstype)
+		return fmt.Errorf("%s image mounts are not authorized", fstype)
 	}
 	if sizelimit == 0 {
 		return fmt.Errorf("invalid image size, zero length")
@@ -694,7 +694,7 @@ func (p *Points) AddOverlay(tag AuthorizedTag, dest string, flags uintptr, lower
 		if !strings.HasPrefix(workdir, "/") {
 			return fmt.Errorf("workdir must be an absolute path")
 		}
-		options = fmt.Sprintf("lowerdir=%s,upperdir=%s,workdir=%s", lowerdir, upperdir, workdir)
+		options = fmt.Sprintf("lowerdir=%s,upperdir=%s,workdir=%s,xino=on", lowerdir, upperdir, workdir)
 	} else {
 		options = fmt.Sprintf("lowerdir=%s", lowerdir)
 	}
@@ -778,3 +778,9 @@ func (p *Points) SetContext(context string) error {
 func (p *Points) GetContext() string {
 	return p.context
 }
+
+// AuthorizeImageFS adds the specified filesystem from the authorizedImage list.
+// This means a loopback mount can then be performed from an image file with this filesystem.
+func AuthorizeImageFS(fs string) {
+	authorizedImage[fs] = fsContext{true}
+}
